use axum::response::IntoResponse;
use http::StatusCode;
use sqlx::{Pool, Postgres};
use thiserror::Error;

use crate::model::{Chat, Message};

type Result<T> = std::result::Result<T, AppStateError>;
const DB_URL: &str = "postgres://chatdings@localhost/chatdings";

#[derive(Debug, Clone)]
pub struct AppState {
    pool: Pool<Postgres>,
}

impl AppState {
    pub async fn init() -> Result<Self> {
        let pool = Pool::<Postgres>::connect(DB_URL).await?;

        sqlx::migrate!()
            .run(&pool)
            .await
            .expect("migration should not fail");

        Ok(Self { pool })
    }

    pub async fn fetch_chat_by_url_path(&self, url_path: &str) -> Result<Chat> {
        Ok(
            sqlx::query_as!(Chat, r#"select * from chats where url_path = $1"#, url_path)
                .fetch_one(&self.pool)
                .await?,
        )
    }

    pub async fn fetch_messages(&self, chat: &Chat) -> Result<Vec<Message>> {
        Ok(sqlx::query_as!(
            Message,
            r#"select * from messages where chat_id = $1"#,
            chat.id
        )
        .fetch_all(&self.pool)
        .await?)
    }

    pub async fn send_message(&self, chat: &Chat, content: String, from_admin: bool) -> Result<()> {
        todo!()
    }

    pub fn pool(&self) -> &Pool<Postgres> {
        &self.pool
    }
}

#[derive(Error, Debug)]
pub enum AppStateError {
    #[error("database error")]
    Sqlx(#[from] sqlx::Error),
}

impl IntoResponse for AppStateError {
    fn into_response(self) -> axum::response::Response {
        todo!()
    }
}

impl From<AppStateError> for StatusCode {
    fn from(value: AppStateError) -> Self {
        match value {
            AppStateError::Sqlx(sqlx::Error::RowNotFound) => Self::NOT_FOUND,
            AppStateError::Sqlx(_) => Self::INTERNAL_SERVER_ERROR,
        }
    }
}
